"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var defaultValue = {
  hh: "00",
  mm: "00",
  ss: "00"
};
var hhInMilli = 3600000,
    mmInMilli = 60000,
    ssInMilli = 1000;

function Timer() {
  this.timerDetails = {};

  this.notifier = function () {};

  this.clientData = {};
}

Timer.timeObjToMillis = function () {
  var duration = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _duration$hh = duration.hh,
      hh = _duration$hh === void 0 ? "00" : _duration$hh,
      _duration$mm = duration.mm,
      mm = _duration$mm === void 0 ? "00" : _duration$mm,
      _duration$ss = duration.ss,
      ss = _duration$ss === void 0 ? "00" : _duration$ss;
  return Number(hh) * hhInMilli + Number(mm) * mmInMilli + Number(ss) * ssInMilli;
};

Timer.millisToTimeObj = function (millis) {
  var hh = "".concat(Math.floor(millis / hhInMilli)),
      mm = "".concat(Math.floor(millis % hhInMilli / mmInMilli)),
      ss = "".concat(Math.floor(millis % hhInMilli % mmInMilli / ssInMilli));
  hh = hh.length === 1 ? "0".concat(hh) : hh;
  mm = mm.length === 1 ? "0".concat(mm) : mm;
  ss = ss.length === 1 ? "0".concat(ss) : ss;
  return {
    hh: hh,
    mm: mm,
    ss: ss
  };
};

Timer.prototype = {
  init: function init(details) {
    var _this = this;

    var data = details.data,
        notifier = details.notifier;
    this.initTimerDetails(data);
    this.initClientData(data);
    this.setNotifier(notifier);
    Object.keys(data).map(function (detailKey) {
      var _data$detailKey = data[detailKey],
          time = _data$detailKey.time,
          status = _data$detailKey.status;

      _this.updateTimerState(detailKey, undefined, time, status);

      _this.deleteTimerDetailKey(detailKey, "isInitCall");
    });
  },
  initClientData: function initClientData(data) {
    var _this2 = this;

    Object.keys(data).map(function (detailKey) {
      _this2.clientData[detailKey] = {};
    });
  },
  setNotifier: function setNotifier(func) {
    this.notifier = func;
  },
  clearInterval: function (_clearInterval) {
    function clearInterval(_x) {
      return _clearInterval.apply(this, arguments);
    }

    clearInterval.toString = function () {
      return _clearInterval.toString();
    };

    return clearInterval;
  }(function (key) {
    var timerDetail = this.getTimerDetail(key);
    clearInterval(timerDetail.intervalId);
    delete timerDetail.intervalId;
  }),
  initTimerDetails: function initTimerDetails(details) {
    var _this3 = this;

    Object.keys(details).map(function (detailKey) {
      _this3.timerDetails[detailKey] = {
        isInitCall: true
      };
    });
  },
  getTimerDetail: function getTimerDetail(key) {
    return this.timerDetails[key];
  },
  notify: function notify(key) {
    this.notifier(key, this.clientData[key]);
  },
  setTimerDetail: function setTimerDetail(key, obj) {
    this.timerDetails[key] = Object.assign({}, this.timerDetails[key], obj);
  },
  deleteTimerDetailKey: function deleteTimerDetailKey(key, detailKey) {
    var newObj = Object.assign({}, this.timerDetails[key]);
    delete newObj[detailKey];
    this.timerDetails[key] = newObj;
  },
  setClientData: function setClientData(key, obj) {
    var _this4 = this;

    var newData = {};
    Object.keys(this.clientData).map(function (clientKey) {
      newData[clientKey] = clientKey !== key ? _this4.clientData[clientKey] : Object.assign({}, _this4.clientData[key], obj);
    });
    this.clientData = newData;
  },
  updateTimerState: function updateTimerState(key, action) {
    var duration = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultValue;
    var status = arguments.length > 3 ? arguments[3] : undefined;

    if (action === "START") {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.start(key);
    } else if (action === "PAUSE" || status === 'PAUSED') {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.pause(key);
    } else if (action === "RESUME" || status === 'RUNNING') {
      this.setTimerDetail(key, {
        previousTimeObj: duration
      });
      this.resume(key);
    } else if (action === "INIT" || action === "STOP" || action === "RESET" || status == 'INIT') {
      this.reset(key);
    }
  },
  triggerTimerMechanism: function triggerTimerMechanism(key) {
    var _this5 = this;

    var intervalId = setInterval(function () {
      var _this5$getTimerDetail = _this5.getTimerDetail(key),
          startTime = _this5$getTimerDetail.startTime;

      var time = Timer.millisToTimeObj(new Date().getTime() - startTime);

      _this5.setClientData(key, {
        time: time
      });

      _this5.notify(key);
    }, 1000);
    this.setTimerDetail(key, {
      intervalId: intervalId
    });
  },
  start: function start(key) {
    this.clearInterval(key);

    var _this$getTimerDetail = this.getTimerDetail(key),
        previousTimeObj = _this$getTimerDetail.previousTimeObj,
        isInitCall = _this$getTimerDetail.isInitCall;

    var startTime = new Date().getTime() - Timer.timeObjToMillis(previousTimeObj);
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.setTimerDetail(key, {
      startTime: startTime
    });
    this.setClientData(key, {
      status: "RUNNING",
      time: Timer.millisToTimeObj(new Date().getTime() - startTime)
    });
    this.triggerTimerMechanism(key);
    isInitCall != true && this.notify(key);
  },
  pause: function pause(key) {
    this.clearInterval(key);
    var currentTime = new Date().getTime();

    var _this$getTimerDetail2 = this.getTimerDetail(key),
        _this$getTimerDetail3 = _this$getTimerDetail2.startTime,
        startTime = _this$getTimerDetail3 === void 0 ? currentTime : _this$getTimerDetail3,
        isInitCall = _this$getTimerDetail2.isInitCall,
        previousTimeObj = _this$getTimerDetail2.previousTimeObj;

    var previousTime = Timer.millisToTimeObj(previousTimeObj ? Timer.timeObjToMillis(previousTimeObj) : currentTime - startTime);
    this.setTimerDetail(key, {
      previousTimeObj: previousTime
    });
    this.setClientData(key, {
      status: "PAUSED",
      time: previousTime
    });
    isInitCall != true && this.notify(key);
  },
  resume: function resume(key) {
    this.clearInterval(key);

    var _this$getTimerDetail4 = this.getTimerDetail(key),
        previousTimeObj = _this$getTimerDetail4.previousTimeObj,
        isInitCall = _this$getTimerDetail4.isInitCall,
        currentTime = new Date().getTime();

    var startTime = currentTime - Timer.timeObjToMillis(previousTimeObj);
    this.setTimerDetail(key, {
      startTime: startTime
    });
    this.setClientData(key, {
      status: "RUNNING",
      time: previousTimeObj
    });
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.triggerTimerMechanism(key);
    isInitCall != true && this.notify(key);
  },
  reset: function reset(key) {
    this.clearInterval(key);
    this.setClientData(key, {
      status: "INIT",
      time: {
        hh: "00",
        mm: "00",
        ss: "00"
      }
    });
    this.deleteTimerDetailKey(key, "previousTimeObj");
    this.deleteTimerDetailKey(key, "startTime");
    this.getTimerDetail(key).isInitCall != true && this.notify(key);
  }
};
var _default = Timer;
exports["default"] = _default;